/*--------------------------------------------------------------------------
 *  Copyright 2008 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-core Project
//
// GenomeWindow.java
// Since: Jan 22, 2008
//
// $URL: http://svn.utgenome.org/utgb/tags/utgb/utgb-core/utgb-core-1.0.3/src/main/java/org/utgenome/graphics/GenomeWindow.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.graphics;

public class GenomeWindow
{
    private final long startIndexOnGenome;
    private final long endIndexOnGenome;
    private long range;
    private boolean isReverse = false;

    public GenomeWindow(long startIndexOnGenome, long endIndexOnGenome)
    {
        this.startIndexOnGenome = startIndexOnGenome;
        this.endIndexOnGenome = endIndexOnGenome;
        // reverse ?
        if (startIndexOnGenome > endIndexOnGenome)
            this.isReverse = true;
        // inclusive
        range = width(endIndexOnGenome, startIndexOnGenome) + 1;
    }

    public static long width(long x1, long x2)
    {
        return (x1 < x2) ? x2 - x1 : x1 - x2;
    }

    public int getXPosOnWindow(long indexOnGenome, int canvasWidth)
    {
        double v = (indexOnGenome - startIndexOnGenome) * (double) canvasWidth;

        if (isReverse)
            v = canvasWidth - v;

        double v2 = v / (double) range;
        return (int) v2;
    }

    public long getGenomeRange()
    {
        return range;
    }

    public boolean getReverse()
    {
        return isReverse;
    }

    public boolean hasOverlap(long startOnGenome, long endOnGenome)
    {
        if (isReverse)
        {
            return startOnGenome >= this.endIndexOnGenome && endOnGenome <= this.startIndexOnGenome;
        }
        else
        {
            return startOnGenome <= this.endIndexOnGenome && endOnGenome >= this.startIndexOnGenome;
        }
    }
}
